// SafeList.inl : implementation file
//


/////////////////////////////////////////////////////////////////////////////
// CSafeList

template<class TYPE, class ARG_TYPE>
inline CSafeList<TYPE, ARG_TYPE>::CSafeList(const CSafeList& listSrc)
{
   m_pcsListAccess = NULL;
   m_pcsListAccess = new CRITICAL_SECTION;
   ::InitializeCriticalSection(m_pcsListAccess);

   POSITION pos = listSrc.GetHeadPosition();
  
   ARG_TYPE* pData;
   while(NULL != pos)
   {
      pData = new ARG_TYPE;
      *pData = listSrc.GetNext(pos);
      AddTail(*pData);
   }
}

template<class TYPE, class ARG_TYPE>
inline void CSafeList<TYPE, ARG_TYPE>::Lock()const
{
   ::EnterCriticalSection(m_pcsListAccess);
}

template<class TYPE, class ARG_TYPE>
inline void CSafeList<TYPE, ARG_TYPE>::Unlock()const
{
   ::LeaveCriticalSection(m_pcsListAccess);
}

template<class TYPE, class ARG_TYPE>
inline CSafeList<TYPE, ARG_TYPE>::CSafeList(int nBlockSize):m_List(nBlockSize)
{
   m_pcsListAccess = NULL;
   m_pcsListAccess = new CRITICAL_SECTION;
   ::InitializeCriticalSection(m_pcsListAccess);
}

template<class TYPE, class ARG_TYPE>
inline CSafeList<TYPE, ARG_TYPE>::~CSafeList()
{
   ASSERT(m_pcsListAccess);
   ::DeleteCriticalSection(m_pcsListAccess);
   delete m_pcsListAccess;
}

template<class TYPE, class ARG_TYPE>
void inline CSafeList<TYPE, ARG_TYPE>::RemoveAll()
{ 
   Lock();
   m_List.RemoveAll();  
   Unlock();
}

template<class TYPE, class ARG_TYPE>
void inline CSafeList<TYPE, ARG_TYPE>::RemoveAt(POSITION position)
{
   Lock();
   m_List.RemoveAt(position);  
   Unlock();
}

template<class TYPE, class ARG_TYPE>
inline TYPE CSafeList<TYPE, ARG_TYPE>::GetNext(POSITION& rPosition) const // return *Position++
{ 
   Lock();
   TYPE type = m_List.GetNext(rPosition);
   Unlock();
   return type;
}
/*
template<class TYPE, class ARG_TYPE>
inline TYPE& inline CSafeList<TYPE, ARG_TYPE>::GetNext(POSITION& rPosition) // return *Position++
{ 
   Lock()
   TYPE type = m_List.GetNext(rPosition);
   Unlock();
   return type;
}
*/

template<class TYPE, class ARG_TYPE>
inline POSITION CSafeList<TYPE, ARG_TYPE>::GetHeadPosition() const
{ 
   Lock();
   POSITION pos = m_List.GetHeadPosition();
   Unlock();
   return pos;
}

template<class TYPE, class ARG_TYPE>
inline BOOL CSafeList<TYPE, ARG_TYPE>::IsEmpty() const
{ 
   Lock();
   BOOL b = m_List.IsEmpty();
   Unlock();
   return b;
}

template<class TYPE, class ARG_TYPE>
inline POSITION CSafeList<TYPE, ARG_TYPE>::AddHead(ARG_TYPE newElement)
{
   Lock();
   POSITION pos = m_List.AddHead(newElement);  
   Unlock();
   return pos;
}


template<class TYPE, class ARG_TYPE>
inline int CSafeList<TYPE, ARG_TYPE>::GetCount() const
{ 
   Lock();
   int i = m_List.GetCount();
   Unlock();
   return i;
}

/*
template<class TYPE, class ARG_TYPE>
inline TYPE& inline CSafeList<TYPE, ARG_TYPE>::GetHead()
{ 
   Lock();
   CNode data = m_List.GetHead();
   Unlock();
   return data;
}
template<class TYPE, class ARG_TYPE>
inline TYPE& inline CSafeList<TYPE, ARG_TYPE>::GetTail()
{ 
   Lock();
   CNode data = m_List.GetTail();
   Unlock();
   return data;
}
template<class TYPE, class ARG_TYPE>
inline TYPE& inline CSafeList<TYPE, ARG_TYPE>::GetPrev(POSITION& rPosition) // return *Position--
{ 
   Lock();
   CNode data = m_List.GetPrev(rPosition);
   Unlock();
   return data;
}
template<class TYPE, class ARG_TYPE>
inline TYPE& inline CSafeList<TYPE, ARG_TYPE>::GetAt(POSITION position)
{ 
   Lock();
   CNode data = m_List.GetAt(rPosition);
   Unlock();
   return data;
}


*/
template<class TYPE, class ARG_TYPE>
inline TYPE CSafeList<TYPE, ARG_TYPE>::GetHead() const
{ 
   Lock();
  // CNode data = m_List.GetHead();
   TYPE data = m_List.GetHead();
   Unlock();
   return data;
}

template<class TYPE, class ARG_TYPE>
inline TYPE CSafeList<TYPE, ARG_TYPE>::GetTail() const
{ 
   Lock();
   CNode data = m_List.GetTail();
   Unlock();
   return data;
}


template<class TYPE, class ARG_TYPE>
inline POSITION CSafeList<TYPE, ARG_TYPE>::GetTailPosition() const
{ 
   Lock();
   POSITION pos = m_List.GetTailPosition();
   Unlock();
   return pos;
}

template<class TYPE, class ARG_TYPE>
inline TYPE CSafeList<TYPE, ARG_TYPE>::GetPrev(POSITION& rPosition) const // return *Position--
{ 
   Lock();
   CNode data = m_List.GetPrev(rPosition);
   Unlock();
   return data;
}
template<class TYPE, class ARG_TYPE>
inline TYPE CSafeList<TYPE, ARG_TYPE>::GetAt(POSITION position) const
{ 
   Lock();
   CNode data = m_List.GetAt(rPosition);
   Unlock();
   return data;
}

template<class TYPE, class ARG_TYPE>
inline void CSafeList<TYPE, ARG_TYPE>::SetAt(POSITION pos, ARG_TYPE newElement)
{ 
   Lock();
   m_List.SetAt(pos,newElement);  
   Unlock();
}
template<class TYPE, class ARG_TYPE>
inline POSITION CSafeList<TYPE, ARG_TYPE>::AddTail(ARG_TYPE newElement)
{
   Lock();
   POSITION pos = m_List.AddTail(newElement);  
   Unlock();
   return pos;
}
template<class TYPE, class ARG_TYPE>
inline TYPE CSafeList<TYPE, ARG_TYPE>::RemoveHead()
{
   Lock();
   TYPE type = m_List.RemoveHead();  
   Unlock();
   return type;
}

template<class TYPE, class ARG_TYPE>
inline void CSafeList<TYPE, ARG_TYPE>::AddTail(CSafeList* pNewList)
{
   Lock();
   m_List.AddTail(pNewList);  
   Unlock();
}

template<class TYPE, class ARG_TYPE>
inline void CSafeList<TYPE, ARG_TYPE>::AddHead(CSafeList* pNewList)
{
   Lock();
   m_List.AddHead(pNewList->m_List);  
   Unlock();
}









