// Member functions
//
/////////////////////////////////////////////////////////////////////////
//    Protected member methods for lock/unlock
template<class T>
void SAFETYPE<T>::SingleLock() const
{
   DWORD dwRes = WAIT_FAILED;

   dwRes = ::WaitForSingleObject(m_hMutex,DB_INFINITE);
   switch(dwRes)
   {
      case WAIT_OBJECT_0:
      case WAIT_ABANDONED:  // Got it!
      break;

      case WAIT_FAILED:     // oh, darn.
         dwRes = ::GetLastError();
         //  fall through.
      case WAIT_TIMEOUT:
         ASSERT(FALSE);
      break;
   }
}

template<class T>
void SAFETYPE<T>::DoubleLock(HANDLE hMutex) const
{
   DWORD dwRes = WAIT_FAILED;
   HANDLE mutexArray[2];

   mutexArray[0] = m_hMutex;  // member data
   mutexArray[1] = hMutex;    // parameter data

   dwRes = ::WaitForMultipleObjects(2,mutexArray,TRUE,DB_INFINITE);
   switch(dwRes)
   {
   case WAIT_OBJECT_0 + 0:
   case WAIT_OBJECT_0 + 1:
   case WAIT_ABANDONED_0 + 0:
   case WAIT_ABANDONED_0 + 1:    // Got it!
      break;

      case WAIT_FAILED:             // oh, darn.
         dwRes = ::GetLastError(); 
         //  fall through.
      case WAIT_TIMEOUT:
         ASSERT(FALSE);
      break;
   }
}

template<class T>
void SAFETYPE<T>::SingleUnlock() const
{
   VERIFY(::ReleaseMutex(m_hMutex));
}
template<class T>
void SAFETYPE<T>::DoubleUnlock(HANDLE hMutex) const
{
   VERIFY(::ReleaseMutex(m_hMutex));
   VERIFY(::ReleaseMutex(hMutex));
}
/////////////////////////////////////////////////////////////////////////
//    constructors, destructor
template<class T>
SAFETYPE<T>::SAFETYPE()
{
   m_hMutex = ::CreateMutex(NULL,FALSE,NULL);
   ASSERT(m_hMutex);
}
template<class T>
SAFETYPE<T>::SAFETYPE(const T& t)
{ 
   m_hMutex = ::CreateMutex(NULL,FALSE,NULL);
   ASSERT(m_hMutex);
   m_t = t;   
}
template<class T>
SAFETYPE<T>::SAFETYPE(const SAFETYPE<T>& SAFETYPEsrc)
{
   m_hMutex = ::CreateMutex(NULL,FALSE,NULL);
   ASSERT(m_hMutex);
   m_t = SAFETYPEsrc.m_t;  
}

template<class T>
SAFETYPE<T>::~SAFETYPE()
{
   VERIFY(::CloseHandle(m_hMutex));
}


/////////////////////////////////////////////////////////////////////////
//    casting operator and  assignment operator
template<class T>
SAFETYPE<T>::operator T() const
{
   T t;
   SingleLock();
   t = m_t;  
   SingleUnlock();
   return t;
}

template<class T>
const SAFETYPE<T>& SAFETYPE<T>::operator=(const SAFETYPE<T>& SAFETYPEsrc)
{

   DoubleLock(SAFETYPEsrc.m_hMutex);
   m_t = SAFETYPEsrc.m_t;  
   DoubleUnlock(SAFETYPEsrc.m_hMutex);

   return *this;
}


/////////////////////////////////////////////////////////////////////////
//  member function - unary operators  +, -, !, ~, *, &
//
template<class T>
const SAFETYPE<T> SAFETYPE<T>::operator+()
{
   T t;
   SingleLock();
   t = +m_t;  
   SingleUnlock();
   return t;
}
template<class T>
const SAFETYPE<T> SAFETYPE<T>::operator~()
{
   T t;
   SingleLock();
   t = ~m_t;  
   SingleUnlock();
   return t;
}
template<class T>
const SAFETYPE<T> SAFETYPE<T>::operator-()
{
   T t;
   SingleLock();
   t = -m_t;  
   SingleUnlock();
   return t;
}
template<class T>
BOOL SAFETYPE<T>::operator!()
{
   BOOL b;
   SingleLock();
   b = !(m_t);  
   SingleUnlock();
   return b;
}
// NOBODY CAN ACCESS & or * BECAUSE THEY ARE PROTECTED.  THIS IS INTENTIONAL.
/*
template<class T>
const SAFETYPE<T>& SAFETYPE<T>::operator&()
{
   ASSERT(FALSE);
   return *this;
}
template<class T>
const SAFETYPE<T>& SAFETYPE<T>::operator*()
{
   ASSERT(FALSE);
   return *this;
}
*/
/////////////////////////////////////////////////////////////////////////
//  member function - increment/decrement, postfix and prefix
template<class T>
const SAFETYPE<T>& SAFETYPE<T>::operator++()
{
   SingleLock();
   m_t += 1;  
   SingleUnlock();
   return *this;
}

template<class T>
const SAFETYPE<T> SAFETYPE<T>::operator++(int)
{
   T t;
   SingleLock();
   t = m_t;
   m_t += 1;
   SingleUnlock();
   return t;
}
template<class T>
const SAFETYPE<T>& SAFETYPE<T>::operator--()
{
   SingleLock();
   m_t -= 1;  
   SingleUnlock();
   return *this;
}
template<class T>
const SAFETYPE<T> SAFETYPE<T>::operator--(int)
{
   T t;
   SingleLock();
   t = m_t;
   m_t -= 1;
   SingleUnlock();
   return t;
}

/////////////////////////////////////////////////////////////////////////
// member functions arithmetic binary assignment: +=, -=, *=, /=, %=
template<class T>
const SAFETYPE<T>& SAFETYPE<T>::operator+=(const SAFETYPE<T>& SAFETYPEsrc)
{
   DoubleLock(SAFETYPEsrc.m_hMutex);
   m_t += SAFETYPEsrc.m_t;  
   DoubleUnlock(SAFETYPEsrc.m_hMutex);
   return *this;
}
template<class T>
const SAFETYPE<T>& SAFETYPE<T>::operator-=(const SAFETYPE<T>& SAFETYPEsrc)
{
   DoubleLock(SAFETYPEsrc.m_hMutex);
   m_t -= SAFETYPEsrc.m_t;  
   DoubleUnlock(SAFETYPEsrc.m_hMutex);
   return *this;
}
template<class T>
const SAFETYPE<T>& SAFETYPE<T>::operator*=(const SAFETYPE<T>& SAFETYPEsrc)
{
   DoubleLock(SAFETYPEsrc.m_hMutex);
   m_t *= SAFETYPEsrc.m_t;  
   DoubleUnlock(SAFETYPEsrc.m_hMutex);
   return *this;
}
template<class T>
const SAFETYPE<T>& SAFETYPE<T>::operator/=(const SAFETYPE<T>& SAFETYPEsrc)
{
   DoubleLock(SAFETYPEsrc.m_hMutex);
   m_t /= SAFETYPEsrc.m_t;  
   DoubleUnlock(SAFETYPEsrc.m_hMutex);
   return *this;
}
template<class T>
const SAFETYPE<T>& SAFETYPE<T>::operator%=(const SAFETYPE<T>& SAFETYPEsrc)
{
   DoubleLock(SAFETYPEsrc.m_hMutex);
   m_t %= SAFETYPEsrc.m_t;  
   DoubleUnlock(SAFETYPEsrc.m_hMutex);
   return *this;
}

/////////////////////////////////////////////////////////////////////////
// member functions arithmetic binary assignment: &=, |=, ^=, >>=, <<=
template<class T>
const SAFETYPE<T>& SAFETYPE<T>::operator&=(const SAFETYPE<T>& SAFETYPEsrc)
{
   DoubleLock(SAFETYPEsrc.m_hMutex);
   m_t &= SAFETYPEsrc.m_t;  
   DoubleUnlock(SAFETYPEsrc.m_hMutex);
   return *this;
}
template<class T>
const SAFETYPE<T>& SAFETYPE<T>::operator|=(const SAFETYPE<T>& SAFETYPEsrc)
{
   DoubleLock(SAFETYPEsrc.m_hMutex);
   m_t |= SAFETYPEsrc.m_t;  
   DoubleUnlock(SAFETYPEsrc.m_hMutex);
   return *this;
}
template<class T>
const SAFETYPE<T>& SAFETYPE<T>::operator^=(const SAFETYPE<T>& SAFETYPEsrc)
{
   DoubleLock(SAFETYPEsrc.m_hMutex);
   m_t ^= SAFETYPEsrc.m_t;  
   DoubleUnlock(SAFETYPEsrc.m_hMutex);
   return *this;
}
template<class T>
const SAFETYPE<T>& SAFETYPE<T>::operator>>=(const SAFETYPE<T>& SAFETYPEsrc)
{
   DoubleLock(SAFETYPEsrc.m_hMutex);
   m_t >>= SAFETYPEsrc.m_t;  
   DoubleUnlock(SAFETYPEsrc.m_hMutex);
   return *this;
}
template<class T>
const SAFETYPE<T>& SAFETYPE<T>::operator<<=(const SAFETYPE<T>& SAFETYPEsrc)
{
   DoubleLock(SAFETYPEsrc.m_hMutex);
   m_t <<= SAFETYPEsrc.m_t;  
   DoubleUnlock(SAFETYPEsrc.m_hMutex);
   return *this;
}


/////////////////////////////////////////////////////////////////////////
// class global function arithmetic binary operators: +, -, *, /, %
template<class T>
 SAFETYPE<T> AFXAPI operator+(const SAFETYPE<T>& SAFETYPEsrc1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(SAFETYPEsrc1) += SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator+(const T& src1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(src1) += SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator+(const SAFETYPE<T>& SAFETYPEsrc1,const T& src2)
{
   return SAFETYPE<T>(src2) += SAFETYPEsrc1;
}

template<class T>
 SAFETYPE<T> AFXAPI operator-(const SAFETYPE<T>& SAFETYPEsrc1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(SAFETYPEsrc1) -= SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator-(const T& src1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(src1) -= SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator-(const SAFETYPE<T>& SAFETYPEsrc1,const T& src2)
{
   return SAFETYPE<T>(SAFETYPEsrc1) -= SAFETYPE<T>(src2);
}

template<class T>
 SAFETYPE<T> AFXAPI operator*(const SAFETYPE<T>& SAFETYPEsrc1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(SAFETYPEsrc1) *= SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator*(const T& src1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(src1) *= SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator*(const SAFETYPE<T>& SAFETYPEsrc1,const T& src2)
{
   return SAFETYPE<T>(src2) *= SAFETYPEsrc1;
}

template<class T>
 SAFETYPE<T> AFXAPI operator/(const SAFETYPE<T>& SAFETYPEsrc1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(SAFETYPEsrc1) /= SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator/(const T& src1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(src1) /= SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator/(const SAFETYPE<T>& SAFETYPEsrc1,const T& src2)
{
   return SAFETYPE<T>(SAFETYPEsrc1) /= SAFETYPE<T>(src2);
}

template<class T>
 SAFETYPE<T> AFXAPI operator%(const SAFETYPE<T>& SAFETYPEsrc1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(SAFETYPEsrc1) %= SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator%(const T& src1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(src1) %= SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator%(const SAFETYPE<T>& SAFETYPEsrc1,const T& src2)
{
   return SAFETYPE<T>(SAFETYPEsrc1) %= SAFETYPE<T>(src2);
}


/////////////////////////////////////////////////////////////////////////
// global bit-wise binary operators:  &, |, ^, <<, >>
template<class T>
 SAFETYPE<T> AFXAPI operator&(const SAFETYPE<T>& SAFETYPEsrc1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(SAFETYPEsrc1) &= SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator&(const T& src1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(src1) &= SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator&(const SAFETYPE<T>& SAFETYPEsrc1,const T& src2)
{
   return SAFETYPE<T>(SAFETYPEsrc1) &= SAFETYPE<T>(src2);
}

template<class T>
 SAFETYPE<T> AFXAPI operator|(const SAFETYPE<T>& SAFETYPEsrc1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(SAFETYPEsrc1) |= SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator|(const T& src1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(src1) |= SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator|(const SAFETYPE<T>& SAFETYPEsrc1,const T& src2)
{
   return SAFETYPE<T>(SAFETYPEsrc1) |= SAFETYPE<T>(src2);
}

template<class T>
 SAFETYPE<T> AFXAPI operator^(const SAFETYPE<T>& SAFETYPEsrc1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(SAFETYPEsrc1) ^= SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator^(const T& src1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(src1) ^= SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator^(const SAFETYPE<T>& SAFETYPEsrc1,const T& src2)
{
   return SAFETYPE<T>(SAFETYPEsrc1) ^= SAFETYPE<T>(src2);
}

template<class T>
 SAFETYPE<T> AFXAPI operator>>(const SAFETYPE<T>& SAFETYPEsrc1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(SAFETYPEsrc1) >>= SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator>>(const T& src1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(src1) >>= SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator>>(const SAFETYPE<T>& SAFETYPEsrc1,const T& src2)
{
   return SAFETYPE<T>(SAFETYPEsrc1) >>= SAFETYPE<T>(src2) ;
}

template<class T>
 SAFETYPE<T> AFXAPI operator<<(const SAFETYPE<T>& SAFETYPEsrc1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(SAFETYPEsrc1) <<= SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator<<(const T& src1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   return SAFETYPE<T>(src1) <<= SAFETYPEsrc2;
}
template<class T>
 SAFETYPE<T> AFXAPI operator<<(const SAFETYPE<T>& SAFETYPEsrc1,const T& src2)
{
   return SAFETYPE<T>(SAFETYPEsrc1) <<= SAFETYPE<T>(src2) ;
}


/////////////////////////////////////////////////////////////////////////
// Global boolean binary operators:  ==, !=, <, >, <=, >=, &&, ||
template<class T>
BOOL AFXAPI operator==(const SAFETYPE<T>& SAFETYPEsrc1,const SAFETYPE<T>& SAFETYPEsrc2)
{
   BOOL b;
   gDoubleLock(SAFETYPEsrc1.m_hMutex, SAFETYPEsrc2.m_hMutex);
   b = SAFETYPEsrc1.m_t == SAFETYPEsrc2.m_t;  
   gDoubleUnlock(SAFETYPEsrc1.m_hMutex, SAFETYPEsrc2.m_hMutex);
   return b;
}
template<class T>
 BOOL AFXAPI operator==(const SAFETYPE<T>& SAFETYPEsrc1,const T& src2)
{
  BOOL b;
   gSingleLock(SAFETYPEsrc1.m_hMutex);
   b = SAFETYPEsrc1.m_t == src2;  
   gSingleUnlock(SAFETYPEsrc1.m_hMutex);
   return b;
}
template<class T>
 BOOL AFXAPI operator==(const T& src1,const SAFETYPE<T>& SAFETYPEsrc2)
{
  BOOL b;
   gSingleLock(SAFETYPEsrc2.m_hMutex);
   b = src1 == SAFETYPEsrc2.m_t;  
   gSingleUnlock(SAFETYPEsrc2.m_hMutex);
   return b;
}

template<class T>
 BOOL AFXAPI operator!=(const SAFETYPE<T>& SAFETYPEsrc1,const SAFETYPE<T>& SAFETYPEsrc2)
{
  BOOL b;
   gDoubleLock(SAFETYPEsrc1.m_hMutex, SAFETYPEsrc2.m_hMutex);
   b = SAFETYPEsrc1.m_t != SAFETYPEsrc2.m_t;  
   gDoubleUnlock(SAFETYPEsrc1.m_hMutex, SAFETYPEsrc2.m_hMutex);
   return b;
}
template<class T>
 BOOL AFXAPI operator!=(const SAFETYPE<T>& SAFETYPEsrc1,const T& src2)
{
  BOOL b;
   gSingleLock(SAFETYPEsrc1.m_hMutex);
   b = SAFETYPEsrc1.m_t != src2;  
   gSingleUnlock(SAFETYPEsrc1.m_hMutex);
   return b;
}
template<class T>
 BOOL AFXAPI operator!=(const T& src1,const SAFETYPE<T>& SAFETYPEsrc2)
{
  BOOL b;
   gSingleLock(SAFETYPEsrc2.m_hMutex);
   b = src1 != SAFETYPEsrc2.m_t;  
   gSingleUnlock(SAFETYPEsrc2.m_hMutex);
   return b;
}

template<class T>
 BOOL AFXAPI operator>(const SAFETYPE<T>& SAFETYPEsrc1,const SAFETYPE<T>& SAFETYPEsrc2)
{
  BOOL b;
   gDoubleLock(SAFETYPEsrc1.m_hMutex, SAFETYPEsrc2.m_hMutex);
   b = SAFETYPEsrc1.m_t > SAFETYPEsrc2.m_t;  
   gDoubleUnlock(SAFETYPEsrc1.m_hMutex, SAFETYPEsrc2.m_hMutex);
   return b;
}
template<class T>
 BOOL AFXAPI operator>(const SAFETYPE<T>& SAFETYPEsrc1,const T& src2)
{
  BOOL b;
   gSingleLock(SAFETYPEsrc1.m_hMutex);
   b = SAFETYPEsrc1.m_t > src2;  
   gSingleUnlock(SAFETYPEsrc1.m_hMutex);
   return b;
}
template<class T>
 BOOL AFXAPI operator>(const T& src1,const SAFETYPE<T>& SAFETYPEsrc2)
{
  BOOL b;
   gSingleLock(SAFETYPEsrc2.m_hMutex);
   b = src1 > SAFETYPEsrc2.m_t;  
   gSingleUnlock(SAFETYPEsrc2.m_hMutex);
   return b;
}

template<class T>
 BOOL AFXAPI operator<(const SAFETYPE<T>& SAFETYPEsrc1,const SAFETYPE<T>& SAFETYPEsrc2)
{
  BOOL b;
   gDoubleLock(SAFETYPEsrc1.m_hMutex, SAFETYPEsrc2.m_hMutex);
   b = SAFETYPEsrc1.m_t < SAFETYPEsrc2.m_t;  
   gDoubleUnlock(SAFETYPEsrc1.m_hMutex, SAFETYPEsrc2.m_hMutex);
   return b;
}
template<class T>
 BOOL AFXAPI operator<(const SAFETYPE<T>& SAFETYPEsrc1,const T& src2)
{
  BOOL b;
   gSingleLock(SAFETYPEsrc1.m_hMutex);
   b = SAFETYPEsrc1.m_t < src2;  
   gSingleUnlock(SAFETYPEsrc1.m_hMutex);
   return b;
}
template<class T>
 BOOL AFXAPI operator<(const T& src1,const SAFETYPE<T>& SAFETYPEsrc2)
{
  BOOL b;
   gSingleLock(SAFETYPEsrc2.m_hMutex);
   b = src1 < SAFETYPEsrc2.m_t;  
   gSingleUnlock(SAFETYPEsrc2.m_hMutex);
   return b;
}

template<class T>
 BOOL AFXAPI operator>=(const SAFETYPE<T>& SAFETYPEsrc1,const SAFETYPE<T>& SAFETYPEsrc2)
{
  BOOL b;
   gDoubleLock(SAFETYPEsrc1.m_hMutex, SAFETYPEsrc2.m_hMutex);
   b = SAFETYPEsrc1.m_t >= SAFETYPEsrc2.m_t;  
   gDoubleUnlock(SAFETYPEsrc1.m_hMutex, SAFETYPEsrc2.m_hMutex);
   return b;
}
template<class T>
 BOOL AFXAPI operator>=(const SAFETYPE<T>& SAFETYPEsrc1,const T& src2)
{
  BOOL b;
   gSingleLock(SAFETYPEsrc1.m_hMutex);
   b = SAFETYPEsrc1.m_t >= src2;  
   gSingleUnlock(SAFETYPEsrc1.m_hMutex);
   return b;
}
template<class T>
 BOOL AFXAPI operator>=(const T& src1,const SAFETYPE<T>& SAFETYPEsrc2)
{
  BOOL b;
   gSingleLock(SAFETYPEsrc2.m_hMutex);
   b = src1 >= SAFETYPEsrc2.m_t;  
   gSingleUnlock(SAFETYPEsrc2.m_hMutex);
   return b;
}

template<class T>
 BOOL AFXAPI operator<=(const SAFETYPE<T>& SAFETYPEsrc1,const SAFETYPE<T>& SAFETYPEsrc2)
{
  BOOL b;
   gDoubleLock(SAFETYPEsrc1.m_hMutex, SAFETYPEsrc2.m_hMutex);
   b = SAFETYPEsrc1.m_t <= SAFETYPEsrc2.m_t;  
   gDoubleUnlock(SAFETYPEsrc1.m_hMutex, SAFETYPEsrc2.m_hMutex);
   return b;
}
template<class T>
 BOOL AFXAPI operator<=(const SAFETYPE<T>& SAFETYPEsrc1,const T& src2)
{
  BOOL b;
   gSingleLock(SAFETYPEsrc1.m_hMutex);
   b = SAFETYPEsrc1.m_t <= src2;  
   gSingleUnlock(SAFETYPEsrc1.m_hMutex);
   return b;
}
template<class T>
 BOOL AFXAPI operator<=(const T& src1,const SAFETYPE<T>& SAFETYPEsrc2)
{
  BOOL b;
   gSingleLock(SAFETYPEsrc2.m_hMutex);
   b = src1 <= SAFETYPEsrc2.m_t;  
   gSingleUnlock(SAFETYPEsrc2.m_hMutex);
   return b;
}

template<class T>
 BOOL AFXAPI operator&&(const SAFETYPE<T>& SAFETYPEsrc1,const SAFETYPE<T>& SAFETYPEsrc2)
{
  BOOL b;
   gDoubleLock(SAFETYPEsrc1.m_hMutex, SAFETYPEsrc2.m_hMutex);
   b = SAFETYPEsrc1.m_t && SAFETYPEsrc2.m_t;  
   gDoubleUnlock(SAFETYPEsrc1.m_hMutex, SAFETYPEsrc2.m_hMutex);
   return b;
}
template<class T>
 BOOL AFXAPI operator&&(const SAFETYPE<T>& SAFETYPEsrc1,const T& src2)
{
  BOOL b;
   gSingleLock(SAFETYPEsrc1.m_hMutex);
   b = SAFETYPEsrc1.m_t && src2;  
   gSingleUnlock(SAFETYPEsrc1.m_hMutex);
   return b;
}
template<class T>
 BOOL AFXAPI operator&&(const T& src1,const SAFETYPE<T>& SAFETYPEsrc2)
{
  BOOL b;
   gSingleLock(SAFETYPEsrc2.m_hMutex);
   b = src1 && SAFETYPEsrc2.m_t;  
   gSingleUnlock(SAFETYPEsrc2.m_hMutex);
   return b;
}

template<class T>
 BOOL AFXAPI operator||(const SAFETYPE<T>& SAFETYPEsrc1,const SAFETYPE<T>& SAFETYPEsrc2)
{
  BOOL b;
   gDoubleLock(SAFETYPEsrc1.m_hMutex, SAFETYPEsrc2.m_hMutex);
   b = SAFETYPEsrc1.m_t || SAFETYPEsrc2.m_t;  
   gDoubleUnlock(SAFETYPEsrc1.m_hMutex, SAFETYPEsrc2.m_hMutex);
   return b;
}
template<class T>
 BOOL AFXAPI operator||(const SAFETYPE<T>& SAFETYPEsrc1,const T& src2)
{
  BOOL b;
   gSingleLock(SAFETYPEsrc1.m_hMutex);
   b = SAFETYPEsrc1.m_t || src2;  
   gSingleUnlock(SAFETYPEsrc1.m_hMutex);
   return b;
}
template<class T>
 BOOL AFXAPI operator||(const T& src1,const SAFETYPE<T>& SAFETYPEsrc2)
{
  BOOL b;
   gSingleLock(SAFETYPEsrc2.m_hMutex);
   b = src1 || SAFETYPEsrc2.m_t;  
   gSingleUnlock(SAFETYPEsrc2.m_hMutex);
   return b;
}

